#include "monitor10V.h"
#include "../ui_monitor10V.h"


MonitorWidget10V::MonitorWidget10V(QWidget *parent)
    : QWidget(parent), ui(new Ui::MonitorWidget10VClass)
{
    ui->setupUi(this);
    dError = true;
    dOutput = true;

    //ui->plot->setTitle("Real time tracking");
#if QT_VERSION >= 0x040000
#ifdef Q_WS_X11
    /*
       Qt::WA_PaintOnScreen is only supported for X11, but leads
       to substantial bugs with Qt 4.2.x/Windows
     */
    ui->plot->canvas()->setAttribute(Qt::WA_PaintOnScreen, true);
#endif
#endif
    for (int i = 0; i< PLOT_SIZE; i++)
    {
	d_x[i] = i;     // time axis
	d_Err[i] = 0;
	d_Out[i] = 0;
    }

    // Insert new curves
    cErr = new QwtPlotCurve(tr("Error"));
    cErr->attach(ui->plot);

    cOut = new QwtPlotCurve(tr("Output"));
    cOut->attach(ui->plot);

    // Set curve styles
#if QT_VERSION >= 0x040000
	cErr->setRenderHint(QwtPlotItem::RenderAntialiased);
	cOut->setRenderHint(QwtPlotItem::RenderAntialiased);
#endif
    cErr->setPen(QPen(Qt::red));
    cOut->setPen(QPen(Qt::blue));

    // Attach (don't copy) data. Both curves use the same x array.
    cErr->setRawData(d_x, d_Err, PLOT_SIZE);
    cOut->setRawData(d_x, d_Out, PLOT_SIZE);

    // Axis
    ui->plot->setAxisTitle(QwtPlot::xBottom, tr("Time (points)"));
    ui->plot->setAxisScale(QwtPlot::xBottom, 0, PLOT_SIZE);

    ui->plot->setAxisTitle(QwtPlot::yLeft, tr("Values"));
    ui->plot->setAxisScale(QwtPlot::yLeft, -1.5, 1.5);
    ui->plot->setAxisAutoScale(QwtPlot::yLeft);

    ui->plot->insertLegend(new QwtLegend(), QwtPlot::TopLegend);

    dispError(dError);
    dispOutput(dOutput);

    timer = new QTimer;
    timer->setSingleShot(false);
    timer->setInterval(1000);
    connect(timer, SIGNAL(timeout()), this, SLOT(slot_Timeout()));

    connect(ui->checkBoxSampling, SIGNAL(clicked()), this, SLOT(sampleEnableChanged()));
    connect(ui->samplingPeriod, SIGNAL(valueChanged(double)), this, SLOT(samplePeriodChanged(double)));
    connect(ui->pushButtonSampling, SIGNAL(clicked()), this, SLOT(slot_Timeout()));
    connect(ui->StepTestLaunch, SIGNAL(clicked()), this, SLOT(launchStepTest()));
}

MonitorWidget10V::~MonitorWidget10V()
{
    delete ui;
}

void MonitorWidget10V::clearPlot()
{
    ui->plot->clear();
}

void MonitorWidget10V::addPoint(double Error, double Output)
{
    for ( int i = PLOT_SIZE - 1; i > 0; i-- )
    {
	d_Err[i] = d_Err[i-1];
	d_Out[i] = d_Out[i-1];
    }
    d_Err[0] = Error;
    d_Out[0] = Output;
    ui->plot->replot();
}

void MonitorWidget10V::dispError(bool yesno)
{
    if(yesno)
	cErr->attach(ui->plot);
    else
	cErr->detach();
}

void MonitorWidget10V::dispOutput(bool yesno)
{
    if(yesno)
	cOut->attach(ui->plot);
    else
	cOut->detach();
}


void MonitorWidget10V::slot_NewData(QString data)
{
    static bool gotError = false, gotOutput = false;
    static double error=0, output=0;

    if(data.indexOf("feedback") != -1)
        ui->encoder->setText(data.mid(data.indexOf(QRegExp("(-{0,1}\\d+)"))));
    if(data.indexOf("command") != -1)
        ui->command->setText(data.mid(data.indexOf(QRegExp("(-{0,1}\\d+)"))));
    if(data.indexOf("error:") != -1)
	if(data.indexOf("(f)ault") == -1)
	{
            error = data.mid(data.indexOf(QRegExp("(-{0,1}\\d+)"))).toDouble();
	    if(gotError)
		addPoint(error, output);
	    gotError = true;
            ui->error->setText(data.mid(data.indexOf(QRegExp("(-{0,1}\\d+)"), data.indexOf("error:"))));
	}
    if(data.indexOf("error_i:") != -1)
        ui->errorI->setText(data.mid(data.indexOf(QRegExp("(-{0,1}\\d+)"))));
    if(data.indexOf("error_d:") != -1)
        ui->errorD->setText(data.mid(data.indexOf(QRegExp("(-{0,1}\\d+)"))));
    if(data.indexOf("servo enabled") != -1)
        ui->enabled->setText(data.mid(data.indexOf(QRegExp("(\\d+)"))));
    if(data.indexOf("output:") != -1)
    {
        ui->output->setText(data.mid(data.indexOf(QRegExp("(-{0,1}\\d+)"))));
	gotOutput = true;
        output = data.mid(data.indexOf(QRegExp("(-{0,1}\\d+)"))).toDouble();
    }
    if(data.indexOf("limit_state:") != -1)
        ui->outLim->setText(data.mid(data.indexOf(QRegExp("(\\d+)"))));


    if(gotError && gotOutput)
    {
	addPoint(error, output);
	gotError = false;
	gotOutput = false;
    }
}

void MonitorWidget10V::slot_Timeout()
{
    emit sig_SendData(QString("s\r\n"));
}


void MonitorWidget10V::samplePeriodChanged(double val)
{
    timer->setInterval(val);
}

void MonitorWidget10V::sampleEnableChanged()
{
    if(ui->checkBoxSampling->isChecked())
	timer->start(ui->samplingPeriod->value());
    else
	timer->stop();
}

void MonitorWidget10V::launchStepTest()
{
    static bool doNotRemind = false;
    if(doNotRemind == false)
    {
	QMessageBox warning(QMessageBox::Warning, QString(tr("Warning")), QString(tr("Motor will move.<br>Please check that this action will be safe for the machine and the operators! <br>Proceed?")),
			    QMessageBox::YesToAll|QMessageBox::Yes|QMessageBox::Abort, this, 0);
	int ret = warning.exec();
	if(ret == QMessageBox::YesToAll)
	    doNotRemind = true;

	if(ret == QMessageBox::Abort)
	    return;
    }
    QString str('r');
    str.append(QString::number(ui->StepTestPrediv->value()-1));
    str.append('c');
    str.append(QString::number(ui->StepTestValue->value()));
    str.append("ok1\r\n");
    if(ui->StepTestAutoFwRev->isChecked())
	ui->StepTestValue->setValue(-ui->StepTestValue->value());
    emit sig_SendData(str);
}

